'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _watcher = require('./watcher');

var _watcher2 = _interopRequireDefault(_watcher);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Sticky = function (_React$Component) {
  _inherits(Sticky, _React$Component);

  function Sticky(props) {
    _classCallCheck(this, Sticky);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Sticky).call(this, props));

    _this.onScroll = function () {
      var isSticky = _this.stickyConditionsMet();
      var hasChanged = _this.state.isSticky !== isSticky;
      var topCorrection = 0;
      var stickyStyle = {};

      if (isSticky) {
        var offset = _this.context.offset || 0;
        var containerRect = _reactDom2.default.findDOMNode(_this.context.container).getBoundingClientRect();
        stickyStyle = {
          position: 'fixed',
          top: offset,
          left: containerRect.left,
          width: containerRect.width
        };
        topCorrection = _this.state.height;

        var bottomLimit = containerRect.bottom - _this.state.height - _this.props.bottomOffset;
        if (offset > bottomLimit) {
          stickyStyle.top = bottomLimit;
        }
      }

      // Update state
      _this.setState({ isSticky: isSticky, stickyStyle: stickyStyle });
      _this.context.container.updateTopCorrection(topCorrection);

      // Publish sticky state change
      if (hasChanged) _this.props.onStickyStateChange(isSticky);
    };

    _this.onResize = function () {
      _this.updateOrigin();
      // emit a scroll event to re-calculate container top offsets
      Sticky.scrollWatcher.emit();
    };

    _this.state = {
      height: 0,
      stickyStyle: {}
    };
    return _this;
  }

  _createClass(Sticky, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.updateOrigin();
      this.updateHeight();

      Sticky.resizeWatcher.on(this.onResize);
      Sticky.scrollWatcher.on(this.onScroll);
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      Sticky.resizeWatcher.off(this.onResize);
      Sticky.scrollWatcher.off(this.onScroll);
    }

    /*
     * Anytime new props are received, force re-evaluation
     */

  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps() {
      this.updateHeight();
    }
  }, {
    key: 'pageOffset',
    value: function pageOffset() {
      return window.pageYOffset || document.documentElement.scrollTop;
    }

    /*
     * Returns true/false depending on if this should be sticky.
     */

  }, {
    key: 'stickyConditionsMet',
    value: function stickyConditionsMet() {
      var offset = this.pageOffset();
      var origin = this.state.origin - (this.context.offset || 0);
      var containerNode = _reactDom2.default.findDOMNode(this.context.container);

      // check conditions
      var stickyTopConditionsMet = offset >= origin + this.props.topOffset;
      var stickyBottomConditionsMet = offset + this.props.bottomOffset < origin + containerNode.getBoundingClientRect().height;
      return stickyTopConditionsMet && stickyBottomConditionsMet;
    }
  }, {
    key: 'updateOrigin',
    value: function updateOrigin() {
      var node = _reactDom2.default.findDOMNode(this);

      // Do some DOM manipulation to where this element's non-sticky position would be
      var previousPosition = node.style.position;
      node.style.position = '';
      var origin = node.getBoundingClientRect().top + this.pageOffset();
      node.style.position = previousPosition;

      this.setState({ origin: origin });
    }
  }, {
    key: 'updateHeight',
    value: function updateHeight() {
      var height = _reactDom2.default.findDOMNode(this).getBoundingClientRect().height;
      this.setState({ height: height });
    }

    /*
     * The special sauce.
     */

  }, {
    key: 'render',
    value: function render() {
      var className = this.props.className + ' ' + (this.state.isSticky ? this.props.stickyClass : '');

      var style = this.props.style;
      if (this.state.isSticky) {
        style = _extends({}, this.props.style, this.state.stickyStyle, this.props.stickyStyle);
      }

      return _react2.default.createElement(
        'div',
        { className: className, style: style },
        this.props.children
      );
    }
  }]);

  return Sticky;
}(_react2.default.Component);

Sticky.contextTypes = {
  container: _react2.default.PropTypes.any,
  offset: _react2.default.PropTypes.number
};
Sticky.defaultProps = {
  className: '',
  style: {},
  stickyClass: 'sticky',
  stickyStyle: {},
  topOffset: 0,
  bottomOffset: 0,
  onStickyStateChange: function onStickyStateChange() {}
};
Sticky.scrollWatcher = new _watcher2.default(['scroll', 'touchstart', 'touchend']);
Sticky.resizeWatcher = new _watcher2.default(['resize']);
exports.default = Sticky;
module.exports = exports['default'];